# SMP Cluster Julia

The HPE Superdome Flex is a large shared memory node. It is especially well suited for data
intensive application scenarios, for example to process extremely large data sets completely in main
memory or in very fast NVMe memory.

## Becoming a Stand-Alone Cluster

The former HPC system Taurus is partly switched-off and partly split up into separate clusters
until the end of 2023. One such upcoming separate cluster is what you have known as partition
`julia` so far. Since February 2024, `Julia` is now a
stand-alone cluster with

* homogenous hardware resources available at `julia.hpc.tu-dresden.de`,
* and own Slurm batch system.

## Hardware Resources

The hardware specification is documented on the page
[HPC Resources](hardware_overview.md#julia).

## Local Temporary on NVMe Storage

There are 370 TB of NVMe devices installed. For immediate access for all projects, a volume of 87 TB
of fast NVMe storage is available at `/nvme/1/<projectname>`. A quota of
100 GB per project on this NVMe storage is set.

With a more detailed proposal to [hpc-support@tu-dresden.de](mailto:hpc-support@tu-dresden.de)
on how this unique system (large shared memory + NVMe storage) can speed up their computations, a
project's quota can be increased or dedicated volumes of up to the full capacity can be set up.

## Hints for Usage

- Granularity should be a socket (28 cores)
- Can be used for OpenMP applications with large memory demands
- To use Open MPI it is necessary to export the following environment
  variables, so that Open MPI uses shared-memory instead of InfiniBand
  for message transport:

  ```
  export OMPI_MCA_pml=ob1
  export OMPI_MCA_mtl=^mxm
  ```

- Use `I_MPI_FABRICS=shm` so that Intel MPI doesn't even consider
  using InfiniBand devices itself, but only shared-memory instead
